/**
 * @file
 * VuoList_VuoScreen C type definition.
 *
 * @copyright Copyright © 2012–2014 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoScreen_H
#define VuoList_VuoScreen_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoScreen VuoList_VuoScreen
 * A list of @ref VuoScreen elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoScreen_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoScreen elements.
 */
typedef const struct VuoList_VuoScreen_struct { void *l; } * VuoList_VuoScreen;
/// @}
#endif

/**
 * Creates a new list of @ref VuoScreen elements.
 */
VuoList_VuoScreen VuoListCreate_VuoScreen(void);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoScreen VuoListCopy_VuoScreen(const VuoList_VuoScreen list);

/**
 * Returns the @ref VuoScreen at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoScreen VuoListGetValue_VuoScreen(const VuoList_VuoScreen list, const unsigned long index);

/**
 * Changes the @ref VuoScreen at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoScreen(const VuoList_VuoScreen list, const VuoScreen value, const unsigned long index);

/**
 * Inserts the `VuoScreen` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoScreen(const VuoList_VuoScreen list, const VuoScreen value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoScreen(VuoList_VuoScreen list, const VuoScreen value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoScreen(VuoList_VuoScreen list, const VuoScreen value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoScreen(VuoList_VuoScreen list, const unsigned long indexA, const unsigned long indexB);

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoScreen(VuoList_VuoScreen list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoScreen(VuoList_VuoScreen list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoScreen(VuoList_VuoScreen list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoScreen(VuoList_VuoScreen list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoScreen(VuoList_VuoScreen list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoScreen(VuoList_VuoScreen list);

/**
 * Removes the `VuoScreen` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoScreen(VuoList_VuoScreen list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoScreen(const VuoList_VuoScreen list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoScreen VuoList_VuoScreen_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoScreen_getJson(const VuoList_VuoScreen value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoScreen_getSummary(const VuoList_VuoScreen value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoScreen VuoList_VuoScreen_makeFromString(const char *str);
char * VuoList_VuoScreen_getString(const VuoList_VuoScreen value);
/// @}

/**
 * @}
 */

#endif
