/**
 * @file
 * VuoList_VuoTransform C type definition.
 *
 * @copyright Copyright © 2012–2014 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoTransform_H
#define VuoList_VuoTransform_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoTransform VuoList_VuoTransform
 * A list of @ref VuoTransform elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoTransform_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoTransform elements.
 */
typedef const struct VuoList_VuoTransform_struct { void *l; } * VuoList_VuoTransform;
/// @}
#endif

/**
 * Creates a new list of @ref VuoTransform elements.
 */
VuoList_VuoTransform VuoListCreate_VuoTransform(void);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoTransform VuoListCopy_VuoTransform(const VuoList_VuoTransform list);

/**
 * Returns the @ref VuoTransform at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoTransform VuoListGetValue_VuoTransform(const VuoList_VuoTransform list, const unsigned long index);

/**
 * Changes the @ref VuoTransform at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoTransform(const VuoList_VuoTransform list, const VuoTransform value, const unsigned long index);

/**
 * Inserts the `VuoTransform` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoTransform(const VuoList_VuoTransform list, const VuoTransform value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoTransform(VuoList_VuoTransform list, const VuoTransform value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoTransform(VuoList_VuoTransform list, const VuoTransform value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoTransform(VuoList_VuoTransform list, const unsigned long indexA, const unsigned long indexB);

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoTransform(VuoList_VuoTransform list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoTransform(VuoList_VuoTransform list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoTransform(VuoList_VuoTransform list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoTransform(VuoList_VuoTransform list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoTransform(VuoList_VuoTransform list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoTransform(VuoList_VuoTransform list);

/**
 * Removes the `VuoTransform` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoTransform(VuoList_VuoTransform list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoTransform(const VuoList_VuoTransform list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoTransform VuoList_VuoTransform_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoTransform_getJson(const VuoList_VuoTransform value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoTransform_getSummary(const VuoList_VuoTransform value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoTransform VuoList_VuoTransform_makeFromString(const char *str);
char * VuoList_VuoTransform_getString(const VuoList_VuoTransform value);
/// @}

/**
 * @}
 */

#endif
