/**
 * @file
 * VuoList_VuoVideoFrame C type definition.
 *
 * @copyright Copyright © 2012–2014 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoVideoFrame_H
#define VuoList_VuoVideoFrame_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoVideoFrame VuoList_VuoVideoFrame
 * A list of @ref VuoVideoFrame elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoVideoFrame_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoVideoFrame elements.
 */
typedef const struct VuoList_VuoVideoFrame_struct { void *l; } * VuoList_VuoVideoFrame;
/// @}
#endif

/**
 * Creates a new list of @ref VuoVideoFrame elements.
 */
VuoList_VuoVideoFrame VuoListCreate_VuoVideoFrame(void);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoVideoFrame VuoListCopy_VuoVideoFrame(const VuoList_VuoVideoFrame list);

/**
 * Returns the @ref VuoVideoFrame at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoVideoFrame VuoListGetValue_VuoVideoFrame(const VuoList_VuoVideoFrame list, const unsigned long index);

/**
 * Changes the @ref VuoVideoFrame at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoVideoFrame(const VuoList_VuoVideoFrame list, const VuoVideoFrame value, const unsigned long index);

/**
 * Inserts the `VuoVideoFrame` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoVideoFrame(const VuoList_VuoVideoFrame list, const VuoVideoFrame value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoVideoFrame(VuoList_VuoVideoFrame list, const VuoVideoFrame value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoVideoFrame(VuoList_VuoVideoFrame list, const VuoVideoFrame value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoVideoFrame(VuoList_VuoVideoFrame list, const unsigned long indexA, const unsigned long indexB);

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoVideoFrame(VuoList_VuoVideoFrame list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoVideoFrame(VuoList_VuoVideoFrame list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoVideoFrame(VuoList_VuoVideoFrame list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoVideoFrame(VuoList_VuoVideoFrame list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoVideoFrame(VuoList_VuoVideoFrame list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoVideoFrame(VuoList_VuoVideoFrame list);

/**
 * Removes the `VuoVideoFrame` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoVideoFrame(VuoList_VuoVideoFrame list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoVideoFrame(const VuoList_VuoVideoFrame list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoVideoFrame VuoList_VuoVideoFrame_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoVideoFrame_getJson(const VuoList_VuoVideoFrame value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoVideoFrame_getSummary(const VuoList_VuoVideoFrame value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoVideoFrame VuoList_VuoVideoFrame_makeFromString(const char *str);
char * VuoList_VuoVideoFrame_getString(const VuoList_VuoVideoFrame value);
/// @}

/**
 * @}
 */

#endif
